/*
 * HFS (aka MAC) filesystem/handler for MorphOS/AmigaOS
 * Author: Marek 'March' Szyprowski <marek@amiga.pl>
 */

#include <string.h>

#include <exec/types.h>
#include <exec/execbase.h>
#include <dos/dos.h>
#include <dos/filehandler.h>

#include <proto/exec.h>
#include <proto/dos.h>

#include "mac_fs.h"
#include "mac_protos.h"

#include "version.h"

struct ExecBase *SysBase;
struct DosLibrary *DOSBase;
struct Library *UtilityBase;
struct Library *IntuitionBase;
struct Library *LocaleBase;

CONST_STRPTR version = VERSTAG;

void handler(void)
{
	struct Message *startupmsg;
	struct DosPacket *startuppacket;
	LONG err = ERROR_NO_FREE_STORE;
	SysBase = *(VOLATILE APTR*)4;

	glob = AllocVec(sizeof(struct Globals), MEMF_PUBLIC|MEMF_CLEAR);

    glob->ourtask = FindTask(NULL);
	glob->ourport = &((struct Process *)glob->ourtask)->pr_MsgPort;
	WaitPort(glob->ourport);
	startupmsg = GetMsg(glob->ourport);
	startuppacket = (struct DosPacket *) startupmsg->mn_Node.ln_Name;

    glob->devnode = BADDR(startuppacket->dp_Arg3);

	kprintf("\nHFS: opening libraries.\n");
	kprintf("\tFS task: %lx, port %lx, version: " VERS " (" DATE ")\n", (LONG)glob->ourtask, (LONG)glob->ourport);

	if ((DOSBase = (struct DosLibrary*)OpenLibrary("dos.library", 37)))
	{
		if ((IntuitionBase = OpenLibrary("intuition.library", 37)))
		{
			if ((UtilityBase = OpenLibrary("utility.library", 37)))
			{
				if ((LocaleBase = OpenLibrary("locale.library", 53)))
				{
					glob->SysBase = SysBase;
					glob->DOSBase = DOSBase;
					glob->UtilityBase = UtilityBase;
					glob->IntuitionBase = IntuitionBase;

					if ((glob->mempool = CreatePool(MEMF_PUBLIC, DEF_POOL_SIZE, DEF_POOL_TRESHOLD)))
					{
						if ((err = InitFSSM(BADDR(startuppacket->dp_Arg2))) == 0)
						{
							ULONG diskchgsig_bit;
							if ((err = InitDiskHandler(glob->fssm, &diskchgsig_bit)) == 0)
							{
								ULONG pktsig = 1 << glob->ourport->mp_SigBit;
								ULONG diskchgsig = 1 << diskchgsig_bit;
								ULONG mask = pktsig | diskchgsig;
								ULONG sigs;
								struct DosEnvec *de = BADDR(glob->fssm->fssm_Environ);

								glob->buffers_count = de->de_NumBuffers;

								kprintf("\tInitiated device: ");
								knprints(((UBYTE *)BADDR(glob->devnode->dol_Name))+1, ((UBYTE *)BADDR(glob->devnode->dol_Name))[0]);

								LockDosList(LDF_DEVICES | LDF_WRITE);
								glob->devnode->dol_Task = glob->ourport;
								UnLockDosList(LDF_DEVICES | LDF_WRITE);

								ReplyPkt(startuppacket, DOSTRUE, 0);
								kprintf("Handler init finished.\n");

								glob->sb = NULL;
								glob->sblist = NULL;
								glob->disk_inserted = FALSE;
								glob->disk_inhibited = FALSE;
								glob->quit = FALSE;
								glob->autodetect = (de->de_LowCyl == 0);

								/* insert disk */
								ProcessDiskChange();

								/* main loop */
								while (!glob->quit)
								{
									sigs = Wait(mask);

									if (sigs & diskchgsig)
										ProcessDiskChange();
									if (sigs & pktsig)
										ProcessPackets();
								}
								kprintf("\nHandler shutdown initiated\n");
								err = 0;
								startuppacket = NULL;
								CleanupDiskHandler(diskchgsig_bit);
							}
							FreeFSSM();
						}
						DeletePool(glob->mempool);
					}
					else
						err = ERROR_NO_FREE_STORE;

					CloseLibrary(LocaleBase);
				}
				CloseLibrary(UtilityBase);
			}
			CloseLibrary(IntuitionBase);
		}
		CloseLibrary((struct Library*)DOSBase);
	}

	kprintf("The end.\n");
	if (startuppacket)
		ReplyPkt(startuppacket, DOSFALSE, err);
	FreeVec(glob);

	return;
}

